package RPM::Source::TransformContainer;

use strict;
use warnings;

our $VERSION=0.008;

use Source::Shared::CLI 0.009;
use Source::Shared::ResourceContainer;
use Source::Shared::Resource::Verbose 0.001;
use RPM::Source::Transformation::MessageBus;
use RPM::Source::Abstract::PriorityCollection;
our @ISA=qw/Source::Shared::CLI/;
push @Source::Shared::VERBOSE_CLASSES, __PACKAGE__;

our $debug_container;

our @SUPPORT_CLASSES;
our @RESOURCE_CLASSES;
our @FACTORY_CLASSES;
our $PLAYER;
our $WRITER;
our $INPUT_HANDLER;
our $ParentProvider;
our $RPMEditorProvider;

# those are initialized during use in RPM::Source::Transformation::*
BEGIN {
 @SUPPORT_CLASSES=qw//;
 @RESOURCE_CLASSES=qw//;
 @FACTORY_CLASSES=qw//;
 $PLAYER=undef;
 $WRITER=undef;
 $INPUT_HANDLER=undef;
 $ParentProvider=undef;
 $RPMEditorProvider=undef;
}

use constant {
	TRANSFORMATION_FAILURE	=> 101,
	TRANSFORMATION_OK	=>  77,
};

sub debug_container {
    print STDERR "DEBUG:RPM::Source::TransformContainer:","\n";
    print STDERR "DEBUG:SUPPORT_CLASSES:\n\t",join("\n\t", @SUPPORT_CLASSES),"\n";
    print STDERR "DEBUG:RESOURCE_CLASSES:\n\t",join("\n\t", @RESOURCE_CLASSES),"\n";
    print STDERR "DEBUG:FACTORY_CLASSES:\n\t",join("\n\t", @FACTORY_CLASSES),"\n";
    print STDERR "DEBUG:PLAYER=",&__U($PLAYER),"\n";
    print STDERR "DEBUG:WRITER=",&__U($WRITER),"\n";
    print STDERR "DEBUG:INPUT_HANDLER=",&__U($INPUT_HANDLER),"\n";
    print STDERR "DEBUG:ParentProvider=",&__U($ParentProvider),"\n";
    print STDERR "DEBUG:RPMEditorProvider=",&__U($RPMEditorProvider),"\n";
}

sub __U {
    return defined($_[0]) ? $_[0]: '(undef)';
}

sub init_transformations {
    my $resourceobj=Source::Shared::ResourceContainer::get_resourceobj();
    map {$_->init_factory($resourceobj)} @RPM::Source::TransformContainer::FACTORY_CLASSES;
    &debug_container if $debug_container;
}

sub mk_opt {
    my %opt;
    tie %opt, 'RPM::Source::Transformation::MessageBus';
    # @RESOURCE_CLASSES don't register_messages
    map {$_->register_messages(\%opt)} 
        @RPM::Source::TransformContainer::SUPPORT_CLASSES, 
        @RPM::Source::TransformContainer::FACTORY_CLASSES;
    return \%opt;
}

sub transform_once {
    my (undef,$spec,$parent,$opt)=@_;

    my $resourceobj=Source::Shared::ResourceContainer::get_resourceobj();
    map {$_->prepare_resource($spec,$parent,$opt,$resourceobj)} @RPM::Source::TransformContainer::RESOURCE_CLASSES;

    my $actions=RPM::Source::Abstract::PriorityCollection->new();
    $actions->add(map {$_->transformations($opt,$resourceobj)} @RPM::Source::TransformContainer::FACTORY_CLASSES);

    if ($RPM::Source::TransformContainer::PLAYER->apply_collection($spec,$parent,$opt,$resourceobj,$actions) == TRANSFORMATION_FAILURE) {
	goto CLEANUP;
    }

    my $write_rpm_ok;
    if ($RPM::Source::TransformContainer::WRITER) {
	$write_rpm_ok=$RPM::Source::TransformContainer::WRITER->write($spec,$parent,$opt,$resourceobj);
    }
  CLEANUP:
    map {$_->destroy_resource($spec,$parent,$opt,$resourceobj)} @RPM::Source::TransformContainer::RESOURCE_CLASSES;
    $parent->cleanup() if $parent;
    $spec->cleanup() if $write_rpm_ok || (exists($opt->{-cleanup}) and $opt->{-cleanup});
    
}

#-----------------------------------------------
# Let's just inherit methods from CLI container.
#-----------------------------------------------

sub cli_container_ref {
    return ['Source::Shared::ResourceContainer', @SUPPORT_CLASSES, @RESOURCE_CLASSES, @FACTORY_CLASSES];
}

1;
